#
# admin_cgi.py - Admin CGI methods.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: admin_cgi.py,v 1.10 2005/03/20 09:05:27 fuktommy Exp $
#

import cgi
import re

import config
from cache import *
from gateway import CommonGatewayCGI

__version__ = "$Revision: 1.10 $"


class AdminCGI(CommonGatewayCGI):

    """Class for /admin.cgi."""

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage()

        if not re.search(config.admin, self.environ["REMOTE_ADDR"]):
            self.print403()
            return
        elif form.getfirst("cmd", "") == "delete":
            if form.getfirst("file", "") == "":
                self.print404()
            elif form.getfirst("record", "") == "":
                self.print_delete_file(form.getfirst("file", ""))
            else:
                self.print_delete_record(form.getfirst("file", ""),
                                         form.getfirst("record", ""))
        elif form.getfirst("cmd", "") == "xdelete":
            if form.getfirst("file", "") == "":
                self.print404()
            elif form.getfirst("record", "") == "":
                self.do_delete_file(form.getfirst("file", ""))
            else:
                self.do_delete_record(form.getfirst("file", ""),
                                      form.getfirst("record", ""),
                                      form.getfirst("dopost", ""),
                                      form)
        elif path == "index":
            self.print_index()
        elif path == "changes":
            self.print_changes()
        elif path == "update":
            self.print_update()
        elif path.startswith("search"):
            self.print_search(path, form)
        else:
            self.print404()

    def print_delete_record(self, datfile, idstr):
        """Delete record dialog."""
	self.header(self.message["del_record"])
        rec = Record(datfile=datfile, idstr=idstr)
        if not rec.exists():
            self.stdout.write("<p>" + self.message["no_record"] + "</p>")
        else:
            self.print_delete_record_form(datfile, rec)
        self.footer()

    def print_delete_record_form(self, datfile, rec):
        rec.load()
        buf = re.sub(r"attach:[^<>]*(<>|$)", "", rec.recstr)
        buf = cgi.escape(buf)
        self.stdout.write(
            "<p>" + self.message["del_record_q"] + "</p>" +
            '<form method="post" action="' + self.admin_cgi + '"><p>' +
            '<input type="hidden" name="cmd" value="xdelete" />' +
            '<input type="hidden" name="file" value="' + rec.datfile + '" />' +
            '<input type="hidden" name="record" value="' + rec.idstr + '" />' +
            '<input type="submit" name="submit" value="' +
            self.message["yes"] + '" name="submit"' +
            ' tabindex="1" accesskey="w" />' +
            ' <input type="checkbox" name="dopost" value="dopost"' +
            ' tabindex="2" accesskey="s" />' + 
            self.message["send"] + "<br />" +
            " " + self.message["name"] +
            ':<input name="name" size="15" value=""' +
            ' tabindex="3" accesskey="n" />' +
            " " + self.message["signature"] +
            ':<input type="password" name="passwd" size="15"' +
            ' value="" tabindex="4" accesskey="p" />' +
            " " + self.message["comment"] + 
            ':<input name="message" size="15" value=""' +
            ' tabindex="5" accesskey="m" />' +
            "</p></form>\n" +
            "<p>" + buf + "</p>\n")

    def do_delete_record(self, datfile, idstr, dopost="", form=None):
        rec = Record(datfile=datfile, idstr=idstr)
        size = rec.size()
        removed = rec.remove()
        cache = Cache(datfile)
        cache.sync_body_light(size=size)
        title = self.str_encode(self.file_decode(datfile))
        for type in config.types:
            if datfile.startswith(type + "_"):
                next = self.appli[type] + "?" + title
                break
        else:
            next = self.root

        if removed and (dopost != ""):
            self.post_delete_message(cache, rec, form)

        self.print302(next)

    def post_delete_message(self, cache, rec, form):
        """Post delete message to other nodes."""
        stamp = self.error_time()

        body = {}
        for key in ("name", "message"):
            if form.getfirst(key, "") != "":
                body[key] = self.escape(form.getfirst(key, ""))
        body["remove_stamp"] = str(rec.stamp)
        body["remove_id"] = rec.id
        passwd = form.getfirst("passwd", "")
        id = rec.build(stamp, body, passwd=passwd)
        cache.add_data([rec])

        updatelist = UpdateList()
        updatelist.append(rec)
        updatelist.sync()
        nodelist = NodeList()
        nodelist.tell_update(datfile=cache.datfile, stamp=stamp, id=id)

    def print_delete_file(self, datfile):
        """Delete file dialog."""
        cache = Cache(datfile)
	self.header(self.message["del_file"])
        if cache.exists():
            self.print_delete_file_form(cache)
        else:
            self.stdout.write("<p>%s: %s</p>" %
                              (datfile, self.message["no_file"]))
        self.footer()

    def print_delete_file_form(self, cache):
        self.stdout.write(
            "<p>" + self.message["del_file_q"] +
            ' <a href="' + self.admin_cgi + '?cmd=xdelete&amp;file=' +
            cache.datfile + '">' +
            self.message["yes"] + "</a></p>\n")
        for type in config.types:
            if cache.datfile.startswith(type + "_"):
                mode = self.message[type]
                title = self.file_decode(cache.datfile)
                break
        else:
            mode = ""
            title = cache.datfile
        self.stdout.write("<p>%s(%s)</p>\n" % (title, mode))
        count = 0
        for idstr in cache:
            count += 1
            rec = cache[idstr]
            rec.load()
            buf = re.sub(r"attach:[^<>]*(<>|$)", "", rec.recstr)
            buf = cgi.escape(buf)
            self.stdout.write("<p>%s<p>" % buf)
            if (count > 2):
                break

    def do_delete_file(self, datfile):
        cache = Cache(datfile)
        cache.remove()
        self.print302(self.list_cgi)

    def print_index_list(self, cachelist, footer=True):
        """Print index list."""
        self.stdout.write('<form method="get" action="' +
                          self.admin_cgi + '">\n')
        for type in config.types:
            self.stdout.write(
                "<h2>" + self.message[type] + "</h2>\n" +
                '<p>\n' +
                '  <input type="hidden" name="cmd" value="delete" />\n' +
                '</p>\n' +
                "<ul>\n")
                
            for cache in cachelist:
                if cache.type != type:
                    continue
                x = self.file_decode(cache.datfile)
                y = self.str_encode(x)
                self.stdout.write(
                    "  <li>\n" +
                    '    <input type="radio" name="file" value="' +
                    cache.datfile + '" tabindex="1" accesskey="s" />\n' +
                    self.localtime(cache.stamp) + " " +
                    '    <a href="' + self.appli[type] + "?" +
                    y + '">' + x + '</a>\n' +
                    '    (' + str(len(cache)) + "/" +
                    str(int(cache.size/1024/1024*10)/10.0) +
                    self.message["mb"] + ")\n" +
                    "  </li>\n")
            self.stdout.write("</ul>\n")
        self.stdout.write(
            '<p><input type="submit" value="' +
            self.message["del_file"] + '" tabindex="2" accesskey="d" />' +
            '</p>\n' +
            "</form>\n")
        if footer:
            self.print_new_element_form();
            self.footer()

    def print_index(self):
        """Print index page."""
        self.header(self.message["index"])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(a.datfile, b.datfile))
        self.print_index_list(cachelist)

    def print_changes(self):
        """Print changes page."""
        self.header(self.message["changes"])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        self.print_index_list(cachelist)

    def print_update(self):
        """Print changes page."""
        self.header(self.message["update"])
        updatelist = UpdateList()
        updatelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        cachelist = []
        check = []
        for rec in updatelist:
            if rec.datfile not in check:
                cache = Cache(rec.datfile)
                cache.stamp = rec.stamp
                cachelist.append(cache)
                check.append(rec.datfile)
        self.print_index_list(cachelist)

    def print_search_form(self, query=""):
        self.stdout.write(
            '<form method="get" action="' + self.admin_cgi + '/search"><p>\n'+
            '  <input name="query" size="19" value="' + query + '"' +
            ' tabindex="1" accesskey="q" />\n' +
            '  <input type="submit" value="' + self.message["search"] + '"' +
            ' name="submit" tabindex="1" accesskey="s" />\n' +
            '</p></form>\n')

        self.stdout.write(
            '<form method="get" action="' + config.google + '"><p>\n' +
            '  <input name="q" size="19" value="' + query + '"'
            ' tabindex="1" accesskey="g" />\n' +
            '  <input type="hidden" name="ie" value="UTF-8" />\n' +
            '  <input type="submit" name="google" value="' +
            self.message["google"] + '" tabindex="1" accesskey="g" />\n' +
            "</p></form>\n")

    def print_search_result(self, query):
        self.header(self.message["search"])
        cachelist = CacheList().search(query)
        self.print_index_list(cachelist, footer=False)
        self.print_search_form(query)
        self.footer()

    def print_search(self, path, form):
        query = cgi.escape(form.getfirst("query", ""), True)
        if query == "":
            query = path[len("search/"):].replace('"', "&quote;")

        if query == "":
            query = self.environ.get("QUERY_STRING", "")
            query = self.escape(self.str_decode(query))

        if query == "":
            self.header(self.message["search"])
            self.print_search_form()
            self.footer()
        else:
            self.print_search_result(query)
