#
# client_cgi.py - Client CGI methods.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: client_cgi.py,v 1.7 2005/03/18 10:26:01 fuktommy Exp $
#

import re
import sys
import time

import config
from basecgi import *
from cache import *
from node import *

__version__ = "$Revision: 1.7 $"


class Status(dict):

    """Time stamp for client process.

    format:
        ping
        init
        sync
    """

    statusfile = config.client_log

    def __init__(self):
        dict.__init__(self)
        self.update({"ping": 0, "init": 0, "sync": 0})
        try:
            f = file(self.statusfile)
            for k in ("ping", "init", "sync"):
                self[k] = int(f.readline())
            f.close()
        except IOError:
            sys.stderr.write(self.statusfile + ": IOError\n")
        except ValueError:
            sys.stderr.write("Wrong format\n")

    def sync(self):
        try:
            f = file(self.statusfile, "wb")
            for k in ("ping", "init", "sync"):
                f.write(str(self[k]) + "\n")
            f.close()
        except IOError:
            sys.stderr.write(self.statusfile + ": IOError\n")

    def check(self, key):
        """Task is done."""
        self[key] = str(int(time.time()))

# Status


class ClientCGI(CGI):

    """Class for /client.cgi."""

    def run(self):    
        self.header("text/plain")
        if not re.search(config.admin, self.environ["REMOTE_ADDR"]):
            self.stdout.write("You are not the administrator.\n")
            self.stdout.close()
            return
        self.stdout.close()
        status = Status()
        self.timelimit = int(time.time()) + config.client_timeout

        if int(time.time()) - status["ping"] >= config.ping_cycle:
            self.do_ping()
            status = Status()

        nodelist = NodeList()
        if len(nodelist) == 0:
            self.do_init()
            self.do_sync()
            status = Status()

        if int(time.time()) - status["init"] >= config.init_cycle * len(nodelist):
            self.do_init()
            status = Status()

        if int(time.time()) - status["sync"] >= config.sync_cycle:
            self.do_sync()

    def check(self, key):
        status = Status()
        status.check(key)
        status.sync()

    def do_ping(self):
        self.check("ping")
        nodelist = NodeList()
        nodelist.pingall()
        nodelist.sync()
        self.stderr.write("shingetsu.node.NodeList.pingall() finished\n")

    def do_init(self):
        self.check("init")
        nodelist = NodeList()
        nodelist.init()
        nodelist.sync()
        searchlist = SearchList()
        searchlist.extend(nodelist)
        searchlist.sync()
        self.stderr.write("shingetsu.node.NodeList.init() finished\n")

    def do_sync(self):
        self.check("sync")
        nodelist = NodeList()
        for n in nodelist[0:]:
            nodelist.join(n)
        nodelist.sync()

        cachelist = CacheList()
        cachelist.clean_history()
        self.stderr.write("shingetsu.cache.CacheList.clean_history()" +
                          " finished\n")

        cachelist.remove_removed()
        self.stderr.write("shingetsu.cache.CacheList.remove_removed()" +
                          " finished\n")

        cachelist.sync_body()
        self.stderr.write("shingetsu.cache.CacheList.sync_body() finished\n")

        cachelist.getall(timelimit=self.timelimit)
        self.stderr.write("shingetsu.cache.CacheList.getall() finished\n")
