#
# gateway_cgi.py - Gateway CGI methods.
# Copyright (C) 2005  shinGETsu Project.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: gateway_cgi.py,v 1.10 2005/03/20 10:38:36 fuktommy Exp $
#

import cgi
import time

import config
from cache import *
from gateway import CommonGatewayCGI
from rss import RSS, make_rss1, make_rss2

__version__ = "$Revision: 1.10 $"


class GatewayCGI(CommonGatewayCGI):

    """Class for /gateway.cgi."""

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage()

        if path == "new":
            self.header(self.message["new"])
            self.print_new_element_form()
            self.footer()
        elif path == "rss":
            self.print_rss()
        elif form.getfirst("cmd", "") == "new":
            self.jump_new_file(form)
        elif path == "list":
            self.print302(self.list_cgi)
        elif re.search(r"^(list|thread|note)", path):
            m = re.search(r"^(list|thread|note)/?([^/]*)$", path)
            if m is None:
                self.print_title()
                return
            elif m.group(2) != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.str_encode(m.group(2))
            elif self.environ.get("QUERY_STRING", "") != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.environ["QUERY_STRING"]
            else:
                self.print_title()
                return

            self.print302(uri)
        else:
            self.print_title()

    def print_title(self):
        message = self.message
        self.header(message["logo"] + " - " + message["description"])
        self.stdout.write(
            "<p>" + message["license"] + "</p>\n" +
            "<ul>\n" +
            '  <li><em><a href="' + self.list_cgi + '">' +
            message["menu"] + "</a></em></li>\n" +
            '  <li><a href="http://shingetsu.sourceforge.net/">' +
	    message["site"] + "</a>\n" +
            '      (<a href="http://shingetsu.xrea.jp/">' +
            message["mirror"] + "</a>)</li>\n" +
            '  <li><a href="' + self.gateway_cgi + '/motd">' +
            message["agreement"] + "</a></li>\n" +
            "</ul>\n" +
	    "<h2>" + message["recent"] + "</h2>\n<ul>\n")

        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        now = int(time.time())
        for cache in cachelist:
            if cache.stamp + config.rss_range < now:
                continue
            date = self.localtime(cache.stamp)
            size = int(cache.size/1024/1024*10)/10.0

            self.stdout.write(
                "  <li>\n" +
                "    " + date + "\n" +
                '    <a href="' + self.appli[cache.type] + self.sep +
                self.str_encode(self.file_decode(cache.datfile)) + '">' +
                self.file_decode(cache.datfile) + "</a>\n" +
		"    (" + message[cache.type] + "/" +
                str(len(cache)) + "/" + str(size) +message["mb"] + ")\n" +
                "  </li>\n")
        self.stdout.write("</ul>\n")
        self.footer()


    def jump_new_file(self, form):
        if form.getfirst("name", "") == "":
            self.header(self.message["null_name"])
            self.footer()
        elif re.search(r"/|\]\]", form.getfirst("name", "")):
            self.header(self.message["bad_name"])
            self.footer()
        elif form.getfirst("type", "") == "":
            self.header(self.message["null_type"])
            self.footer()
        elif form.getfirst("type", "") in config.types:
            self.print302(self.appli[form.getfirst("type", "")] + self.sep +
                          self.str_encode(form.getfirst("name", "")))
        else:
            self.print404()

    def print_rss(self):
        rss = RSS(encode = "UTF-8",
                  title = self.message["logo"],
                  parent = "http://" + self.environ["HTTP_HOST"],
                  uri = "http://" + self.environ["HTTP_HOST"]
                                  + self.gateway_cgi + "/rss",
                  description = self.message["description"])
        cachelist = CacheList()
        now = int(time.time())
        for cache in cachelist:
            if cache.stamp + config.rss_range >= now:
                title = self.file_decode(cache.datfile)
                rss.append((self.appli[cache.type] + self.sep +
                           self.str_encode(title))[1:],
                           date = cache.stamp,
                           title = title)

        self.stdout.write("Content-Type: text/xml; charset=UTF-8\n\n")
        if config.rss == "1":
            self.stdout.write(make_rss1(rss))
        else:
            self.stdout.write(make_rss2(rss))
